import {
  ArrowDownOutlined,
  ArrowUpOutlined,
  DeleteOutlined,
  EditOutlined,
  PlusOutlined,
} from "@ant-design/icons";
import {
  Avatar,
  Button,
  Card,
  Collapse,
  Empty,
  Form,
  Grid,
  Skeleton,
  Space,
  Tooltip,
} from "antd";
import { useTranslation } from "react-i18next";
import { useEffect, useState } from "react";
import { Constants, myFetch } from "../../../utils";
import { useParams } from "react-router-dom";
import MyModal, {
  MyModalCloseCreateButtonFooter,
  MyModalCloseSaveButtonFooter,
} from "../../../Components/MyModal";
import { MyFormInput } from "../../../Components/MyFormInputs";

const { useBreakpoint } = Grid;

export default function StoreServices() {
  const { t } = useTranslation();
  const screenBreakpoint = useBreakpoint();
  const { storeId } = useParams();

  const [isRequestingServices, setIsRequestingServices] = useState(false);
  const [addEditServiceModalOptions, setAddEditServiceModalOptions] = useState({
    mode: "add", // add | edit
    isOpen: false,
    service: null,
  });
  const [
    addEditServiceActivityModalOptions,
    setAddEditServiceActivityModalOptions,
  ] = useState({
    mode: "add", // add | edit
    isOpen: false,
    activity: null,
  });
  const [servicesData, setServicesData] = useState({
    services: [],
    users: [],
  });

  const fetchServices = () => {
    setIsRequestingServices(true);

    myFetch(`/store/services/${storeId}`, "GET")
      .then((data) => {
        setIsRequestingServices(false);
        setServicesData(data);
      })
      .catch((errStatus) => {
        console.log(errStatus);
      });
  };

  useEffect(() => fetchServices(), []);

  return (
    <>
      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
          flexDirection: screenBreakpoint.xs ? "column" : "row",
        }}
      >
        <h1>{t("storeServices.pageTitle")}</h1>

        <ModalAddEditService
          storeId={storeId}
          fetchServices={fetchServices}
          addEditServiceModalOptions={addEditServiceModalOptions}
          setAddEditServiceModalOptions={setAddEditServiceModalOptions}
        />
      </div>
      <Space direction="vertical" style={{ width: "100%" }}>
        {isRequestingServices ? (
          <>
            <Skeleton.Input block active size="large" />
            <Skeleton.Input block active size="large" />
            <Skeleton.Input block active size="large" />
          </>
        ) : (
          <>
            {servicesData.services.map((service) => (
              <Service
                key={service.service_id}
                service={service}
                users={servicesData.users}
                storeId={storeId}
                setAddEditServiceActivityModalOptions={
                  setAddEditServiceActivityModalOptions
                }
                setAddEditServiceModalOptions={setAddEditServiceModalOptions}
              />
            ))}
          </>
        )}
      </Space>

      <ModalAddEditServiceActivity
        fetchServices={fetchServices}
        addEditServiceActivityModalOptions={addEditServiceActivityModalOptions}
        setAddEditServiceActivityModalOptions={
          setAddEditServiceActivityModalOptions
        }
      />
    </>
  );
}

function Service({
  service,
  users,
  storeId,
  setAddEditServiceActivityModalOptions,
  setAddEditServiceModalOptions,
}) {
  const [isRequestingActivities, setIsRequestingActivities] = useState(false);
  const [isOpen, setIsOpen] = useState(false);

  const [serviceActivities, setServiceActivities] = useState([]);

  const fetchServiceActivities = () => {
    if (!isOpen) return;

    setIsRequestingActivities(true);

    myFetch(
      `/store/services/activities/${storeId}/${service.service_id}`,
      "GET"
    )
      .then((data) => {
        setIsRequestingActivities(false);
        setServiceActivities(data.activities);
      })
      .catch((errStatus) => {
        console.log(errStatus);
      });
  };

  useEffect(() => fetchServiceActivities(), [isOpen]);

  return (
    <Collapse
      key={service.service_id}
      onChange={(e) => setIsOpen(e.length !== 0)}
      items={[
        {
          key: "1",
          label: (
            <div
              style={{
                display: "flex",
                justifyContent: "space-between",
                alignItems: "center",
              }}
            >
              <span>{service.name}</span>

              <Space>
                <ArrowUpOutlined
                  disabled
                  onClick={(e) => e.stopPropagation()}
                />
                <ArrowDownOutlined
                  disabled
                  onClick={(e) => e.stopPropagation()}
                />
                <PlusOutlined
                  onClick={(e) => {
                    e.stopPropagation();

                    setAddEditServiceActivityModalOptions({
                      mode: "add",
                      isOpen: true,
                      service: service,
                    });
                  }}
                />
                <EditOutlined
                  onClick={(e) => {
                    e.stopPropagation();

                    setAddEditServiceModalOptions({
                      mode: "edit",
                      isOpen: true,
                      service: service,
                    });
                  }}
                />
                <DeleteOutlined onClick={(e) => e.stopPropagation()} />
              </Space>
            </div>
          ),
          children: (
            <Space
              key={`space-${service.service_id}`}
              direction="vertical"
              style={{ width: "100%" }}
            >
              {isRequestingActivities ? (
                <Skeleton active>
                  <Card title="Test">
                    <p>Desc</p>
                    <p>Preis: €</p>
                    <p>Dauer: Minuten</p>
                  </Card>
                </Skeleton>
              ) : (
                <>
                  {serviceActivities.length === 0 ? (
                    <Empty />
                  ) : (
                    serviceActivities.map((activity) => (
                      <Card
                        key={activity.activity_id}
                        title={activity.name}
                        extra={
                          <Space>
                            <Avatar.Group maxCount={1} size="small">
                              {users.map((user) => (
                                <Tooltip
                                  key={user.user_id}
                                  title={user.username}
                                >
                                  <Avatar
                                    size="small"
                                    style={{ backgroundColor: "#87d068" }}
                                  >
                                    {user.username.charAt(0)}
                                  </Avatar>
                                </Tooltip>
                              ))}
                            </Avatar.Group>

                            <ArrowUpOutlined disabled />
                            <ArrowDownOutlined disabled />
                            <EditOutlined
                              onClick={() => {
                                setAddEditServiceActivityModalOptions({
                                  mode: "edit",
                                  isOpen: true,
                                  activity: activity,
                                });
                              }}
                            />
                            <DeleteOutlined />
                          </Space>
                        }
                      >
                        <p>{activity.description}</p>
                        <p>Preis: {activity.price} €</p>
                        <p>Dauer: {activity.duration} Minuten</p>
                      </Card>
                    ))
                  )}
                </>
              )}
            </Space>
          ),
        },
      ]}
    />
  );
}

// this modal is used to create and edit services
function ModalAddEditService({
  storeId,
  fetchServices,
  addEditServiceModalOptions,
  setAddEditServiceModalOptions,
}) {
  const { t } = useTranslation();
  const screenBreakpoint = useBreakpoint();
  const [form] = Form.useForm();

  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => {
    setAddEditServiceModalOptions({
      ...addEditServiceModalOptions,
      isOpen: false,
    });

    form.resetFields();
  };

  useEffect(() => {
    if (!addEditServiceModalOptions.isOpen) return;

    if (addEditServiceModalOptions.mode === "edit") {
      form.setFieldsValue({
        serviceName: addEditServiceModalOptions.service.name,
      });
    }
  }, [addEditServiceModalOptions.isOpen]);

  return (
    <>
      <Button
        type="primary"
        icon={<PlusOutlined />}
        block={screenBreakpoint.xs}
        onClick={() =>
          setAddEditServiceModalOptions({
            ...addEditServiceModalOptions,
            isOpen: true,
          })
        }
      >
        {t("storeServices.buttonAddService")}
      </Button>

      <MyModal
        title={
          addEditServiceModalOptions.mode === "add"
            ? t("storeServices.modalAddService.title")
            : t("storeServices.modalEditService.title")
        }
        isOpen={addEditServiceModalOptions.isOpen}
        onCancel={handleModalClose}
        footer={
          addEditServiceModalOptions.mode === "add" ? (
            <MyModalCloseCreateButtonFooter
              onCancel={handleModalClose}
              isCreateButtonLoading={isRequesting}
              onCreate={() => {
                form
                  .validateFields()
                  .then((values) => {
                    setIsRequesting(true);

                    myFetch("/store/services", "POST", {
                      storeId: storeId,
                      name: values.serviceName,
                    })
                      .then(() => {
                        setIsRequesting(false);
                        handleModalClose();

                        fetchServices();
                      })
                      .catch((errStatus) => {
                        console.log(errStatus);
                      });
                  })
                  .catch((info) => {
                    console.log("Validate Failed:", info);
                  });
              }}
            />
          ) : (
            <MyModalCloseSaveButtonFooter
              onCancel={handleModalClose}
              isSaveButtonLoading={isRequesting}
              onSave={() => {
                // if the service name didn't change, don't send a request
                let formServiceName = form.getFieldValue("serviceName");

                if (
                  addEditServiceModalOptions.service.name === formServiceName
                ) {
                  handleModalClose();
                  return;
                }
                form
                  .validateFields()
                  .then(() => {
                    setIsRequesting(true);

                    myFetch("/store/services/update", "POST", {
                      serviceId: addEditServiceModalOptions.service.service_id,
                      name: formServiceName,
                    })
                      .then(() => {
                        setIsRequesting(false);
                        handleModalClose();

                        fetchServices();
                      })
                      .catch((errStatus) => {
                        console.log(errStatus);
                      });
                  })
                  .catch((info) => {
                    console.log("Validate Failed:", info);
                  });
              }}
            />
          )
        }
      >
        <Form form={form} layout="vertical" requiredMark={false}>
          <ServiceNameFormInput formItemName="serviceName" />
        </Form>
      </MyModal>
    </>
  );
}

let defaultModalAddServiceFormValidOptions = {
  serviceActivityName: false,
  serviceActivityDescription: false,
  serviceActivityPrice: false,
  serviceActivityDurationHours: false,
  serviceActivityDurationMinutes: false,
};

function ModalAddEditServiceActivity({
  fetchServices,
  addEditServiceActivityModalOptions,
  setAddEditServiceActivityModalOptions,
}) {
  const { t } = useTranslation();

  const [isFormValid, setIsFormValid] = useState(
    defaultModalAddServiceFormValidOptions
  );
  const [form] = Form.useForm();
  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => {
    setAddEditServiceActivityModalOptions({
      ...addEditServiceActivityModalOptions,
      isOpen: false,
    });

    form.resetFields();
  };

  useEffect(() => {
    if (!addEditServiceActivityModalOptions.isOpen) return;

    setIsFormValid(
      addEditServiceActivityModalOptions.mode === "add"
        ? defaultModalAddServiceFormValidOptions
        : {
            serviceActivityName: true,
            serviceActivityDescription: true,
            serviceActivityPrice: true,
            serviceActivityDurationHours: true,
            serviceActivityDurationMinutes: true,
          }
    );

    if (addEditServiceActivityModalOptions.mode === "edit") {
      form.setFieldsValue({
        serviceActivityName: addEditServiceActivityModalOptions.activity.name,
        serviceActivityDescription:
          addEditServiceActivityModalOptions.activity.description,
        serviceActivityPrice: addEditServiceActivityModalOptions.activity.price,
        serviceActivityDurationHours: Math.floor(
          addEditServiceActivityModalOptions.activity.duration / 60
        ),
        serviceActivityDurationMinutes:
          addEditServiceActivityModalOptions.activity.duration % 60,
      });
    }
  }, [addEditServiceActivityModalOptions.isOpen]);

  return (
    <MyModal
      title={
        addEditServiceActivityModalOptions.mode === "add"
          ? t("storeServices.modalAddServiceActivity.title")
          : t("storeServices.modalEditServiceActivity.title")
      }
      isOpen={addEditServiceActivityModalOptions.isOpen}
      onCancel={handleModalClose}
      footer={
        addEditServiceActivityModalOptions.mode === "add" ? (
          <MyModalCloseCreateButtonFooter
            onCancel={handleModalClose}
            isCreateButtonDisabled={
              !isFormValid.serviceActivityName ||
              !isFormValid.serviceActivityDescription ||
              !isFormValid.serviceActivityPrice ||
              !isFormValid.serviceActivityDurationHours ||
              !isFormValid.serviceActivityDurationMinutes
            }
            isCreateButtonLoading={isRequesting}
            onCreate={() => {
              setIsRequesting(true);

              myFetch("/store/services/activity", "POST", {
                serviceId:
                  addEditServiceActivityModalOptions.service.service_id,
                name: form.getFieldValue("serviceActivityName"),
                description: form.getFieldValue("serviceActivityDescription"),
                price: form.getFieldValue("serviceActivityPrice"),
                duration:
                  form.getFieldValue("serviceActivityDurationHours") * 60 +
                  form.getFieldValue("serviceActivityDurationMinutes"),
              })
                .then(() => {
                  setIsRequesting(false);
                  handleModalClose();

                  fetchServices();
                })
                .catch((errStatus) => {
                  console.log(errStatus);
                });
            }}
          />
        ) : (
          <MyModalCloseSaveButtonFooter
            onCancel={handleModalClose}
            isSaveButtonDisabled={
              !isFormValid.serviceActivityName ||
              !isFormValid.serviceActivityDescription ||
              !isFormValid.serviceActivityPrice ||
              !isFormValid.serviceActivityDurationHours ||
              !isFormValid.serviceActivityDurationMinutes
            }
            isSaveButtonLoading={isRequesting}
            onSave={() => {
              // if the service name didn't change, don't send a request
              if (
                addEditServiceActivityModalOptions.activity.name ===
                  form.getFieldValue("serviceActivityName") &&
                addEditServiceActivityModalOptions.activity.description ===
                  form.getFieldValue("serviceActivityDescription") &&
                addEditServiceActivityModalOptions.activity.price ===
                  form.getFieldValue("serviceActivityPrice") &&
                addEditServiceActivityModalOptions.activity.duration ===
                  form.getFieldValue("serviceActivityDurationHours") * 60 +
                    form.getFieldValue("serviceActivityDurationMinutes")
              ) {
                handleModalClose();
                console.log("same");
                return;
              }

              setIsRequesting(true);

              myFetch("/store/services/activity/update", "POST", {
                activityId:
                  addEditServiceActivityModalOptions.activity.activity_id,
                name: form.getFieldValue("serviceActivityName"),
                description: form.getFieldValue("serviceActivityDescription"),
                price: form.getFieldValue("serviceActivityPrice"),
                duration:
                  form.getFieldValue("serviceActivityDurationHours") * 60 +
                  form.getFieldValue("serviceActivityDurationMinutes"),
              })
                .then(() => {
                  setIsRequesting(false);
                  handleModalClose();

                  fetchServices();
                })
                .catch((errStatus) => {
                  console.log(errStatus);
                });
            }}
          />
        )
      }
    >
      <Form form={form} layout="vertical" requiredMark={false}>
        <ServiceActivityNameFormInput
          formItemName="serviceActivityName"
          setIsInputValid={(isValid) =>
            setIsFormValid({
              ...isFormValid,
              serviceActivityName: isValid,
            })
          }
        />

        <ServiceActivityDescriptionFormInput
          formItemName="serviceActivityDescription"
          setIsInputValid={(isValid) =>
            setIsFormValid({
              ...isFormValid,
              serviceActivityDescription: isValid,
            })
          }
        />

        <ServiceActivityPriceFormInput
          formItemName="serviceActivityPrice"
          setIsInputValid={(isValid) =>
            setIsFormValid({
              ...isFormValid,
              serviceActivityPrice: isValid,
            })
          }
        />

        <ServiceActivityDurationHoursFormInput
          formItemName="serviceActivityDurationHours"
          setIsInputValid={(isValid) =>
            setIsFormValid({
              ...isFormValid,
              serviceActivityDurationHours: isValid,
            })
          }
        />

        <ServiceActivityDurationMinutesFormInput
          formItemName="serviceActivityDurationMinutes"
          setIsInputValid={(isValid) =>
            setIsFormValid({
              ...isFormValid,
              serviceActivityDurationMinutes: isValid,
            })
          }
        />
      </Form>
    </MyModal>
  );
}

function ServiceNameFormInput({ formItemName }) {
  const { t } = useTranslation();

  return (
    <MyFormInput
      formItemName={formItemName}
      minLength={Constants.GLOBALS.MIN_STORE_SERVICE_NAME_LENGTH}
      maxLength={Constants.GLOBALS.MAX_STORE_SERVICE_NAME_LENGTH}
      label={t("storeServices.serviceName")}
      ruleMessageValueRequired={t(
        "storeServices.inputRules.serviceNameRequired"
      )}
      ruleMessageValueMinLengthRequired={t(
        "storeServices.inputRules.serviceNameMinLength",
        {
          minLength: Constants.GLOBALS.MIN_STORE_SERVICE_NAME_LENGTH,
        }
      )}
      inputPlaceholder={t("storeServices.serviceNamePlaceholder")}
    />
  );
}

function ServiceActivityNameFormInput({ formItemName, setIsInputValid }) {
  const { t } = useTranslation();

  return (
    <MyFormInput
      formItemName={formItemName}
      setIsInputValid={setIsInputValid}
      minLength={Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_NAME_LENGTH}
      maxLength={Constants.GLOBALS.MAX_STORE_SERVICE_ACTIVITY_NAME_LENGTH}
      label={t("storeServices.serviceActivityName")}
      ruleMessageValueRequired={t(
        "storeServices.inputRules.serviceActivityNameRequired"
      )}
      ruleMessageValueMinLengthRequired={t(
        "storeServices.inputRules.serviceActivityNameMinLength",
        {
          minLength: Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_NAME_LENGTH,
        }
      )}
      inputPlaceholder={t("storeServices.serviceActivityNamePlaceholder")}
    />
  );
}

function ServiceActivityDescriptionFormInput({
  formItemName,
  setIsInputValid,
}) {
  const { t } = useTranslation();

  return (
    <MyFormInput
      inputType="textarea"
      formItemName={formItemName}
      setIsInputValid={setIsInputValid}
      minLength={
        Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_DESCRIPTION_LENGTH
      }
      maxLength={
        Constants.GLOBALS.MAX_STORE_SERVICE_ACTIVITY_DESCRIPTION_LENGTH
      }
      label={t("storeServices.serviceActivityDescription")}
      ruleMessageValueRequired={t(
        "storeServices.inputRules.serviceActivityDescriptionRequired"
      )}
      ruleMessageValueMinLengthRequired={t(
        "storeServices.inputRules.serviceActivityDescriptionMinLength",
        {
          minLength:
            Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_DESCRIPTION_LENGTH,
        }
      )}
      inputPlaceholder={t(
        "storeServices.serviceActivityDescriptionPlaceholder"
      )}
    />
  );
}

function ServiceActivityPriceFormInput({ formItemName, setIsInputValid }) {
  const { t } = useTranslation();

  return (
    <MyFormInput
      propsInput={{
        addonAfter: t("storeServices.serviceActivityPriceUnit"),
      }}
      inputType="number"
      formItemName={formItemName}
      setIsInputValid={setIsInputValid}
      minLength={Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_PRICE}
      maxLength={Constants.GLOBALS.MAX_STORE_SERVICE_ACTIVITY_PRICE}
      label={t("storeServices.serviceActivityPrice")}
      ruleMessageValueRequired={t(
        "storeServices.inputRules.serviceActivityPriceRequired"
      )}
      inputPlaceholder={t("storeServices.serviceActivityPricePlaceholder")}
    />
  );
}

function ServiceActivityDurationHoursFormInput({
  formItemName,
  setIsInputValid,
}) {
  const { t } = useTranslation();

  return (
    <MyFormInput
      propsInput={{
        addonAfter: t("storeServices.serviceActivityDurationHoursUnit"),
      }}
      inputType="number"
      formItemName={formItemName}
      setIsInputValid={setIsInputValid}
      minLength={Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_DURATION_HOURS}
      maxLength={Constants.GLOBALS.MAX_STORE_SERVICE_ACTIVITY_DURATION_HOURS}
      label={t("storeServices.serviceActivityDurationHours")}
      ruleMessageValueRequired={t(
        "storeServices.inputRules.serviceActivityDurationHoursRequired"
      )}
      inputPlaceholder={t(
        "storeServices.serviceActivityDurationHoursPlaceholder"
      )}
    />
  );
}

function ServiceActivityDurationMinutesFormInput({
  formItemName,
  setIsInputValid,
}) {
  const { t } = useTranslation();

  return (
    <MyFormInput
      propsInput={{
        addonAfter: t("storeServices.serviceActivityDurationMinutesUnit"),
      }}
      inputType="number"
      formItemName={formItemName}
      setIsInputValid={setIsInputValid}
      minLength={Constants.GLOBALS.MIN_STORE_SERVICE_ACTIVITY_DURATION_MINUTES}
      maxLength={Constants.GLOBALS.MAX_STORE_SERVICE_ACTIVITY_DURATION_MINUTES}
      label={t("storeServices.serviceActivityDurationMinutes")}
      ruleMessageValueRequired={t(
        "storeServices.inputRules.serviceActivityDurationMinutesRequired"
      )}
      inputPlaceholder={t(
        "storeServices.serviceActivityDurationMinutesPlaceholder"
      )}
    />
  );
}
